#include "tabwidget.h"
#include <config_dialog.h>

TabWidget::TabWidget()
{
	tabbar=new TabBar();
	setTabBar((QTabBar*)tabbar);
	connect(tabbar, SIGNAL(contextMenu(int, const QPoint&)),
			SLOT(onContextMenu(int, const QPoint&)));
	connect(tabbar, SIGNAL(deleteTab(int)),
			SLOT(onDeleteTab(int)));
	connect(tabbar, SIGNAL(wheelEventSignal(QWheelEvent*)),
			SLOT(wheelEvent(QWheelEvent*)));
	connect(tabbar, SIGNAL(moveTab(int, int)),
			SLOT(onMoveTab(int, int)));
}

void TabWidget::closeEvent(QCloseEvent* e)
{
	kdebugf();
	QWidget* current=currentPage();
	delete current;

	if(count()>0)
		e->ignore();
	else
		e->accept();
}

void TabWidget::chatKeyPressed(QKeyEvent* e, CustomInput* k, bool &handled)
{
	handled=true;
	if(HotKey::shortCut(e, "ShortCuts", "MoveTabLeft"))
	{
		if(currentPageIndex()==0)
			onMoveTab(0, count()-1);
		else
			onMoveTab(currentPageIndex(), currentPageIndex()-1);
	}
	else if(HotKey::shortCut(e, "ShortCuts", "MoveTabRight"))
	{
		if(currentPageIndex()==(count()-1))
			onMoveTab(count()-1, 0);
		else
			onMoveTab(currentPageIndex(), currentPageIndex()+1);
	}
	else if(HotKey::shortCut(e, "ShortCuts", "SwitchTabLeft"))
		switchTabLeft();
	else if(HotKey::shortCut(e, "ShortCuts", "SwitchTabRight"))
		switchTabRight();
	else
		handled=false;
}

void TabWidget::onContextMenu(int id, const QPoint& pos)
{
	kdebugf();
	emit contextMenu(page(id), pos);
}

void TabWidget::onMoveTab(int from, int to)
{
	kdebugf();
	qDebug("from==%i, to==%i\n", from, to);
	QString tablabel=label(from);
	QWidget *w=page(from);
	QIconSet tabiconset = tabIconSet(w);
	QString tabtooltip = tabToolTip(w);
	bool current=(w==currentPage());
	blockSignals(true);
	removePage(w);

	insertTab(w, tabiconset, tablabel, to);

	if(current)
		setCurrentPage(to);

	blockSignals(false);
}

void TabWidget::onDeleteTab(int id)
{
	emit deleteTab(page(id));
}

void TabWidget::wheelEvent(QWheelEvent* e)
{
	if(e->delta()>0)
		switchTabLeft();
	else
		switchTabRight();
}

void TabWidget::switchTabLeft()
{
	if(currentPageIndex()==0)
		setCurrentPage(count()-1);
	else
		setCurrentPage(currentPageIndex()-1);
}

void TabWidget::switchTabRight()
{
	if(currentPageIndex()==(count()-1))
		setCurrentPage(0);
	else
		setCurrentPage(currentPageIndex()+1);
}

void TabBar::mousePressEvent(QMouseEvent* e)
{
	QTab* selectedtab=selectTab(e->pos());

	if(selectedtab)
	{
		if(e->button()==LeftButton)
		{
			// zmieniamy kursor na "apk"
			setCursor(QCursor(Qt::SizeAllCursor));
			// zapamitujemy identyfikator karty rdowej
			startid=selectedtab->identifier();
		}
		else if(e->button()==RightButton)
			emit contextMenu(indexOf(selectedtab->identifier()), mapToGlobal(e->pos()));
		else if(e->button()==MidButton)
			emit deleteTab(indexOf(selectedtab->identifier()));
	}

	QTabBar::mousePressEvent(e);
}

void TabBar::mouseReleaseEvent(QMouseEvent* e)
{
	QTab* selectedtab=selectTab(e->pos());

	if(selectedtab)
	{
		// pozycja karty docelowej
		int stopid=selectedtab->identifier();
		// przywracamy kursor
		setCursor(QCursor(Qt::ArrowCursor));

		if(e->button()==LeftButton && indexOf(startid)!=-1 && stopid!=startid)
			// zamieniamy miejcami kart rdow z docelow
			emit moveTab(indexOf(startid), indexOf(stopid));
	}

	QTabBar::mouseReleaseEvent(e);
}

void TabBar::wheelEvent(QWheelEvent *e)
{
	emit wheelEventSignal(e);
}
